@@ -65,7 +65,7 @@ nautilus_navigation_action_get_type (void)
 	static GType type = 0;
 
 	if (type == 0) {
-		static const GTypeInfo type_info = {
+		const GTypeInfo type_info = {
 			sizeof (NautilusNavigationActionClass),
 			(GBaseInitFunc) NULL,
 			(GBaseFinalizeFunc) NULL,
