@@ -53,11 +53,13 @@
 #include <libnautilus-private/nautilus-module.h>
 #include <libnautilus-private/nautilus-file-utilities.h>
 #include <libnautilus-private/nautilus-file-operations.h>
+#include <libnautilus-private/nautilus-trash-monitor.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <libgnomevfs/gnome-vfs-volume-monitor.h>
 
 #include "nautilus-bookmark-list.h"
 #include "nautilus-places-sidebar.h"
+#include "nautilus-window.h"
 
 #define NAUTILUS_PLACES_SIDEBAR_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), NAUTILUS_TYPE_PLACES_SIDEBAR, NautilusPlacesSidebarClass))
 #define NAUTILUS_IS_PLACES_SIDEBAR(obj)         (GTK_CHECK_TYPE ((obj), NAUTILUS_TYPE_PLACES_SIDEBAR))
@@ -85,6 +87,7 @@ typedef struct {
 	GtkWidget *popup_menu_unmount_item;
 	GtkWidget *popup_menu_eject_item;
 	GtkWidget *popup_menu_format_item;
+	GtkWidget *popup_menu_empty_trash_item;
 } NautilusPlacesSidebar;
 
 typedef struct {
@@ -121,6 +124,12 @@ typedef enum {
 static void  nautilus_places_sidebar_iface_init        (NautilusSidebarIface         *iface);
 static void  sidebar_provider_iface_init               (NautilusSidebarProviderIface *iface);
 static GType nautilus_places_sidebar_provider_get_type (void);
+static void  open_selected_bookmark                    (NautilusPlacesSidebar        *sidebar,
+							GtkTreeModel                 *model,
+							GtkTreePath                  *path,
+							gboolean                      open_in_new_window);
+static void  nautilus_places_sidebar_style_set         (GtkWidget                    *widget,
+							GtkStyle                     *previous_style);
 
 /* Identifiers for target types */
 enum {
@@ -343,6 +352,15 @@ update_places (NautilusPlacesSidebar *sidebar)
 	}
 	g_list_free (volumes);
 
+	mount_uri = "trash:///"; /* No need to strdup */
+	icon = nautilus_trash_monitor_is_empty () ? "user-trash" : "user-trash-full";
+	last_iter = add_place (sidebar, PLACES_BUILT_IN,
+			       _("Trash"), icon, mount_uri,
+			       NULL, NULL, 0);
+	if (strcmp (location, mount_uri) == 0) {
+		gtk_tree_selection_select_iter (selection, &last_iter);
+	}
+
 	/* add separator */
 
 	gtk_list_store_append (sidebar->store, &iter);
@@ -459,47 +477,10 @@ row_activated_callback (GtkTreeView *tree_view,
 			GtkTreeViewColumn *column,
 			gpointer user_data)
 {
-	NautilusPlacesSidebar *sidebar;
-	GtkTreeModel *model;
-	GtkTreeIter iter;
-	char *uri;
-	
-	sidebar = NAUTILUS_PLACES_SIDEBAR (user_data);
-	model = gtk_tree_view_get_model (tree_view);
-	
-	if (!gtk_tree_model_get_iter (model, &iter, path)) {
-		return;
-	}
-
-	gtk_tree_model_get 
-		(model, &iter, PLACES_SIDEBAR_COLUMN_URI, &uri, -1);
-	
-	if (uri != NULL) {
-		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-				    "activate from places sidebar window=%p: %s",
-				    sidebar->window, uri);
-		/* Navigate to the clicked location. */
-		nautilus_window_info_open_location
-			(sidebar->window, 
-			 uri, NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE, 0, NULL);
-		g_free (uri);
-	} else {
-		GnomeVFSDrive *drive;
-		gtk_tree_model_get 
-			(model, &iter, PLACES_SIDEBAR_COLUMN_DRIVE, &drive, -1);
-		if (drive != NULL) {
-			char *path;
-
-			path = gnome_vfs_drive_get_device_path (drive);
-			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
-					    "activate drive from places sidebar window=%p: %s",
-					    sidebar->window, path);
-			g_free (path);
-
-			gnome_vfs_drive_mount (drive, volume_op_callback, sidebar);
-			gnome_vfs_drive_unref (drive);
-		}
-	}
+	open_selected_bookmark (NAUTILUS_PLACES_SIDEBAR (user_data),
+				gtk_tree_view_get_model (tree_view),
+				path,
+				FALSE);
 }
 
 static void
@@ -1072,6 +1053,7 @@ bookmarks_popup_menu_detach_cb (GtkWidget *attach_widget,
 	sidebar->popup_menu_unmount_item = NULL;
 	sidebar->popup_menu_eject_item = NULL;
 	sidebar->popup_menu_format_item = NULL;
+	sidebar->popup_menu_empty_trash_item = NULL;
 }
 
 static gboolean
@@ -1130,6 +1112,8 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 	gboolean show_unmount;
 	gboolean show_eject;
 	gboolean show_format;
+	gboolean show_empty_trash;
+	char *uri = NULL;
 	
 	type = PLACES_BUILT_IN;
 
@@ -1142,11 +1126,13 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
 				    PLACES_SIDEBAR_COLUMN_ROW_TYPE, &type,
 				    PLACES_SIDEBAR_COLUMN_DRIVE, &drive,
  				    PLACES_SIDEBAR_COLUMN_VOLUME, &volume,
+				    PLACES_SIDEBAR_COLUMN_URI, &uri,
 				    -1);
 	}
 
 	gtk_widget_set_sensitive (sidebar->popup_menu_remove_item, (type == PLACES_BOOKMARK));
 	gtk_widget_set_sensitive (sidebar->popup_menu_rename_item, (type == PLACES_BOOKMARK));
+	gtk_widget_set_sensitive (sidebar->popup_menu_empty_trash_item, !nautilus_trash_monitor_is_empty ());
 
  	check_visibility (volume, drive,
  			  &show_mount, &show_unmount, &show_eject, &show_format);	
@@ -1156,13 +1142,19 @@ bookmarks_check_popup_sensitivity (NautilusPlacesSidebar *sidebar)
  	if (show_eject) {
  		show_unmount = FALSE;
   	}
+
+	show_empty_trash = (uri != NULL) &&
+			   (!strcmp (uri, "trash:///"));
 	
 	eel_gtk_widget_set_shown (sidebar->popup_menu_separator_item, 
-			show_mount || show_unmount || show_eject || show_format);
+			show_mount || show_unmount || show_eject || show_format || show_empty_trash);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_mount_item, show_mount);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_unmount_item, show_unmount);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_eject_item, show_eject);
 	eel_gtk_widget_set_shown (sidebar->popup_menu_format_item, show_format);
+	eel_gtk_widget_set_shown (sidebar->popup_menu_empty_trash_item, show_empty_trash);
+
+	g_free (uri);
 }
 
 /* Callback used when the selection in the shortcuts tree changes */
@@ -1173,6 +1165,91 @@ bookmarks_selection_changed_cb (GtkTreeSelection      *selection,
 	bookmarks_check_popup_sensitivity (sidebar);
 }
 
+static void
+open_selected_bookmark (NautilusPlacesSidebar *sidebar,
+			GtkTreeModel	      *model,
+			GtkTreePath	      *path,
+			gboolean	      open_in_new_window)
+{
+	GtkTreeIter iter;
+	char *uri;
+
+	if (!path) {
+		return;
+	}
+
+	if (!gtk_tree_model_get_iter (model, &iter, path)) {
+		return;
+	}
+
+	gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_URI, &uri, -1);
+
+	if (uri != NULL) {
+		nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+				    "activate from places sidebar window=%p: %s",
+				    sidebar->window, uri);
+		/* Navigate to the clicked location */
+		if (!open_in_new_window) {
+			nautilus_window_info_open_location (sidebar->window, uri,
+							    NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
+							    0, NULL);
+		} else {
+			NautilusWindow *cur, *new;
+			cur = NAUTILUS_WINDOW (sidebar->window);
+			new = nautilus_application_create_navigation_window (cur->application,
+									     NULL,
+									     gtk_window_get_screen (GTK_WINDOW (cur)));
+			nautilus_window_go_to (new, uri);
+		}
+		g_free (uri);
+
+	} else {
+		GnomeVFSDrive *drive;
+		gtk_tree_model_get (model, &iter, PLACES_SIDEBAR_COLUMN_DRIVE, &drive, -1);
+		if (drive != NULL) {
+			char *path;
+
+			path = gnome_vfs_drive_get_device_path (drive);
+			nautilus_debug_log (FALSE, NAUTILUS_DEBUG_LOG_DOMAIN_USER,
+					    "activate drive from places sidebar window=%p: %s",
+					    sidebar->window, path);
+			g_free (path);
+
+			gnome_vfs_drive_mount (drive, volume_op_callback, sidebar);
+			gnome_vfs_drive_unref (drive);
+		}
+	}
+}
+
+static void
+open_shortcut_from_menu (NautilusPlacesSidebar *sidebar,
+			 gboolean	       open_in_new_window)
+{
+	GtkTreeModel *model;
+	GtkTreePath *path;
+
+	model = gtk_tree_view_get_model (sidebar->tree_view);
+	gtk_tree_view_get_cursor (sidebar->tree_view, &path, NULL);
+
+	open_selected_bookmark (sidebar, model, path, open_in_new_window);
+
+	gtk_tree_path_free (path);
+}
+
+static void
+open_shortcut_cb (GtkMenuItem		*item,
+		  NautilusPlacesSidebar	*sidebar)
+{
+	open_shortcut_from_menu (sidebar, FALSE);
+}
+
+static void
+open_shortcut_in_new_window_cb (GtkMenuItem	      *item,
+				NautilusPlacesSidebar *sidebar)
+{
+	open_shortcut_from_menu (sidebar, TRUE);
+}
+
 /* Rename the selected bookmark */
 static void
 rename_selected_bookmark (NautilusPlacesSidebar *sidebar)
@@ -1276,9 +1353,11 @@ unmount_shortcut_cb (GtkMenuItem           *item,
 			    -1);
 
 	if (volume != NULL) {
-		gnome_vfs_volume_unmount (volume, volume_op_callback, sidebar);
+		nautilus_file_operations_unmount_volume (GTK_WIDGET (sidebar->tree_view),
+				volume, volume_op_callback, sidebar);
 	} else if (drive != NULL) {
-		gnome_vfs_drive_unmount (drive, volume_op_callback, sidebar);
+		nautilus_file_operations_unmount_drive (GTK_WIDGET (sidebar->tree_view),
+				drive, volume_op_callback, sidebar);
 	}
 	gnome_vfs_volume_unref (volume);
 	gnome_vfs_drive_unref (drive);
@@ -1317,6 +1396,13 @@ format_shortcut_cb (GtkMenuItem           *item,
 	g_spawn_command_line_async ("gfloppy", NULL);
 }
 
+static void
+empty_trash_cb (GtkMenuItem           *item,
+		NautilusPlacesSidebar *sidebar)
+{
+	nautilus_file_operations_empty_trash (GTK_WIDGET (sidebar->window));
+}
+
 /* Handler for GtkWidget::key-press-event on the shortcuts list */
 static gboolean
 bookmarks_key_press_event_cb (GtkWidget             *widget,
@@ -1359,6 +1445,20 @@ bookmarks_build_popup_menu (NautilusPlacesSidebar *sidebar)
 			           GTK_WIDGET (sidebar),
 			           bookmarks_popup_menu_detach_cb);
 	
+	item = gtk_image_menu_item_new_with_mnemonic (_("_Open"));
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
+				       gtk_image_new_from_stock (GTK_STOCK_OPEN, GTK_ICON_SIZE_MENU));
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (open_shortcut_cb), sidebar);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
+
+	item = gtk_menu_item_new_with_mnemonic (_("Open in New _Window"));
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (open_shortcut_in_new_window_cb), sidebar);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
+
 	item = gtk_image_menu_item_new_with_label (_("Remove"));
 	sidebar->popup_menu_remove_item = item;
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item),
@@ -1408,6 +1508,15 @@ bookmarks_build_popup_menu (NautilusPlacesSidebar *sidebar)
 	gtk_widget_show (item);
 	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
 
+	/* Empty Trash menu item */
+
+	item = gtk_menu_item_new_with_mnemonic (_("Empty _Trash"));
+	sidebar->popup_menu_empty_trash_item = item;
+	g_signal_connect (item, "activate",
+		    G_CALLBACK (empty_trash_cb), sidebar);
+	gtk_widget_show (item);
+	gtk_menu_shell_append (GTK_MENU_SHELL (sidebar->popup_menu), item);
+
 	bookmarks_check_popup_sensitivity (sidebar);
 }
 
@@ -1486,6 +1595,21 @@ bookmarks_editing_canceled (GtkCellRenderer       *cell,
 	g_object_set (cell, "editable", FALSE, NULL);
 }
 
+static void
+trash_state_changed_cb (NautilusTrashMonitor *trash_monitor,
+			gboolean             state,
+			gpointer             data)
+{
+	NautilusPlacesSidebar *sidebar;
+
+	sidebar = NAUTILUS_PLACES_SIDEBAR (data);
+
+	/* The trash icon changed, update the sidebar */
+	update_places (sidebar);
+
+	bookmarks_check_popup_sensitivity (sidebar);
+}
+
 static void
 nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 {
@@ -1598,6 +1722,11 @@ nautilus_places_sidebar_init (NautilusPlacesSidebar *sidebar)
 						  desktop_location_changed_callback,
 						  sidebar,
 						  G_OBJECT (sidebar));
+
+	g_signal_connect_object (nautilus_trash_monitor_get (),
+				 "trash_state_changed",
+				 G_CALLBACK (trash_state_changed_cb),
+				 sidebar, 0);
 }
 
 static void
@@ -1628,6 +1757,8 @@ static void
 nautilus_places_sidebar_class_init (NautilusPlacesSidebarClass *class)
 {
 	G_OBJECT_CLASS (class)->finalize = nautilus_places_sidebar_finalize;
+
+	GTK_WIDGET_CLASS (class)->style_set = nautilus_places_sidebar_style_set;
 }
 
 static const char *
@@ -1704,6 +1835,17 @@ nautilus_places_sidebar_set_parent_window (NautilusPlacesSidebar *sidebar,
 	update_places (sidebar);
 }
 
+static void
+nautilus_places_sidebar_style_set (GtkWidget *widget,
+				   GtkStyle  *previous_style)
+{
+	NautilusPlacesSidebar *sidebar;
+
+	sidebar = NAUTILUS_PLACES_SIDEBAR (widget);
+
+	update_places (sidebar);
+}
+
 static NautilusSidebar *
 nautilus_places_sidebar_create (NautilusSidebarProvider *provider,
 				 NautilusWindowInfo *window)
