@@ -209,7 +209,7 @@ nautilus_query_editor_class_init (NautilusQueryEditorClass *class)
 	signals[CANCEL] =
 		g_signal_new ("cancel",
 		              G_TYPE_FROM_CLASS (class),
-		              G_SIGNAL_RUN_LAST | GTK_RUN_ACTION,
+		              G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		              G_STRUCT_OFFSET (NautilusQueryEditorClass, cancel),
 		              NULL, NULL,
 		              g_cclosure_marshal_VOID__VOID,
